@@ -1,37 +1,3 @@
-#!/usr/bin/env python3
-
-################################################################
-# readgridded.py
-#
-# model data reading class
-#
-# this file is part of the aerocom_pt package
-#
-#################################################################
-# Created 20171030 by Jan Griesfeller for Met Norway
-#
-# Last changed: See git log
-#################################################################
-
-#Copyright (C) 2017 met.no
-#Contact information:
-#Norwegian Meteorological Institute
-#Box 43 Blindern
-#0313 OSLO
-#NORWAY
-#E-mail: jan.griesfeller@met.no
-#This program is free software; you can redistribute it and/or modify
-#it under the terms of the GNU General Public License as published by
-#the Free Software Foundation; either version 3 of the License, or
-#(at your option) any later version.
-#This program is distributed in the hope that it will be useful,
-#but WITHOUT ANY WARRANTY; without even the implied warranty of
-#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#GNU General Public License for more details.
-#You should have received a copy of the GNU General Public License
-#along with this program; if not, write to the Free Software
-#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#MA 02110-1301, USA
 from collections import OrderedDict as od
 
 import fnmatch
@@ -42,20 +8,18 @@ from pathlib import Path
 import numpy as np
 import pandas as pd
 import iris
+import xarray as xr
 
 from pyaerocom import const, print_log, logger
 from pyaerocom.metastandards import AerocomDataID
-from pyaerocom.variable import Variable, is_3d
+from pyaerocom.variable import Variable
 from pyaerocom.tstype import TsType
 from pyaerocom.io.aux_read_cubes import (compute_angstrom_coeff_cubes,
                                          multiply_cubes,
                                          divide_cubes,
                                          subtract_cubes,
                                          add_cubes,
-                                         mmr_from_vmr,
-                                         rho_from_ts_ps,
-                                         conc_from_vmr,
-                                         conc_from_vmr_STP)
+                                         mmr_from_vmr)
 
 from pyaerocom.helpers import (to_pandas_timestamp,
                                sort_ts_types,
@@ -75,17 +39,12 @@ from pyaerocom.io import AerocomBrowser
 from pyaerocom.io.iris_io import load_cubes_custom, concatenate_iris_cubes
 from pyaerocom.io.helpers import add_file_to_log
 from pyaerocom.griddeddata import GriddedData
-from pyaerocom.units_helpers import compute_concprcp_from_pr_and_wetdep
+from pyaerocom._concprcp_units_helpers import \
+    compute_concprcp_from_pr_and_wetdep
+
 
 class ReadGridded(object):
-    """Class for reading gridded files based on network or model ID
-
-    Note
-    ----
-    The reading only works if files are stored using a valid file naming
-    convention. See package data file `file_conventions.ini <http://
-    aerocom.met.no/pyaerocom/config_files.html#file-conventions>`__ for valid
-    keys. You may define your own fileconvention in this file, if you wish.
+    """Class for reading gridded files using AeroCom file conventions
 
     Attributes
     ----------
@@ -178,7 +137,7 @@ class ReadGridded(object):
                 'dryoa'         :   add_cubes,
                 'sc550dryaer'   :   subtract_cubes,
                 'conc*'         :   multiply_cubes,
-                'mmr*'              :   mmr_from_vmr,
+                'mmr*'          :   mmr_from_vmr,
                 'concox'        :   add_cubes,
                 'vmrox'         :   add_cubes,
                 'fmf550aer'     :   divide_cubes,
@@ -211,9 +170,7 @@ class ReadGridded(object):
 
     VERT_ALT = {'Surface' : 'ModelLevel'}
 
-    def __init__(self, data_id=None, data_dir=None,
-                 file_convention="aerocom3",
-                 init=True):
+    def __init__(self,data_id=None,data_dir=None,file_convention="aerocom3"):
 
         self._data_dir = None
 
@@ -258,7 +215,7 @@ class ReadGridded(object):
                 print_log.warning(repr(e))
 
     @property
-    def data_id(self):
+    def data_id(self) -> str:
         """
         Data ID of dataset
         """
@@ -273,7 +230,7 @@ class ReadGridded(object):
         self._data_id = val
 
     @property
-    def data_dir(self):
+    def data_dir(self) -> str:
         """
         Directory where data files are located
         """
@@ -290,7 +247,7 @@ class ReadGridded(object):
         self.reinit()
 
     @property
-    def years_avail(self):
+    def years_avail(self) -> list:
         """
         Years available in dataset
         """
@@ -299,17 +256,7 @@ class ReadGridded(object):
         return sorted(self.file_info.year.unique())
 
     @property
-    def years(self):
-        """
-        Wrapper for :attr:`years_available`
-        """
-        print_log.warning(DeprecationWarning('Attr. "years" is deprecated '
-                                             '(but still works). Please use '
-                                             '"years_avail" instead.'))
-        return self.years_avail
-
-    @property
-    def experiments(self):
+    def experiments(self) -> list:
         """
         List of all experiments that are available in this dataset
         """
@@ -318,7 +265,7 @@ class ReadGridded(object):
         return sorted(self.file_info.experiment.unique())
 
     @property
-    def files(self):
+    def files(self) -> list:
         """
         List of data files
         """
@@ -411,6 +358,7 @@ class ReadGridded(object):
         self._vars_3d =[]
 
     def _get_vars_provided(self):
+        """(Private method) get all variables provided"""
         _vars = []
         _vars.extend(self.vars_filename)
 
@@ -754,63 +702,15 @@ class ReadGridded(object):
                                 'registered pyaerocom file convention'
                                 .format(self.data_dir))
 
-    def search_all_files(self, update_file_convention=True):
-        """Search all valid model files for this model
-
-        This method browses the data directory and finds all valid files, that
-        is, file that are named according to one of the aerocom file naming
-        conventions. The file list is stored in :attr:`files`.
-
-        Note
-        ----
-        It is presumed, that naming conventions of files in
-        the data directory are not mixed but all correspond to either of the
-        conventions defined in
-
-        Parameters
-        ----------
-        update_file_convention : bool
-            if True, the first file in `data_dir` is used to identify the
-            file naming convention (cf. :class:`FileConventionRead`)
-
-        Raises
-        ------
-        DataCoverageError
-            if no valid files could be found
-        """
-        if self.data_dir is None:
-            raise AttributeError('Cannot search files since :attr:`data_dir` '
-                                 'is not assigned')
-
+    def _evaluate_fileinfo(self, files):
         result = []
-        files_ignored = []
-        # get all netcdf files in folder
-        nc_files = glob(self.data_dir + '/*{}'.format(self.file_type))
-        if len(nc_files) == 0:
-            print_log.warning('No files of type {} could be found in current '
-                              'data_dir={}'.format(self.file_type,
-                                        os.path.abspath(self.data_dir)))
-            return
-
-        if update_file_convention:
-            # Check if the found file has a naming according the aerocom conventions
-            # and set the convention for all files (maybe this need to be
-            # updated in case there can be more than one file naming convention
-            # within one model directory)
-            try:
-                self._update_file_convention(nc_files)
-            except FileNotFoundError as e:
-                print_log.warning(repr(e))
-                return
-
         _vars_temp = []
         _vars_temp_3d = []
 
-        for _file in nc_files:
+        for _file in files:
             # TODO: resolve this in a more general way...
             if 'ModelLevelAtStations' in _file:
                 const.logger.info('Ignoring file {}'.format(_file))
-                files_ignored.append(os.path.basename(_file))
                 continue
             try:
                 info = self.file_convention.get_info_from_file(_file)
@@ -838,20 +738,20 @@ class ReadGridded(object):
 
             except (FileConventionError, DataSourceError,
                     TemporalResolutionError) as e:
-                msg = ("Failed to import file {}\nModel: {}\n"
-                       "Error: {}".format(os.path.basename(_file),
-                                         self.data_id, repr(e)))
+                msg = (f'Failed to import file\n{_file}\nModel: '
+                       f'{self.data_id}\nError: {e}')
                 logger.warning(msg)
                 if const.WRITE_FILEIO_ERR_LOG:
                     add_file_to_log(_file, msg)
 
         if len(_vars_temp + _vars_temp_3d) == 0:
             raise AttributeError("Failed to extract information from filenames")
-        # make sorted list of unique vars
 
         self._vars_2d = sorted(od.fromkeys(_vars_temp))
         self._vars_3d = sorted(od.fromkeys(_vars_temp_3d))
+        return result
 
+    def _fileinfo_to_dataframe(self, result):
         header = ['var_name', 'year', 'ts_type', 'vert_code', 'data_id', 'name',
                   'meteo', 'experiment', 'perturbation', 'is_at_stations',
                   '3D', 'filename']
@@ -865,10 +765,63 @@ class ReadGridded(object):
         uv = df.vert_code.unique()
         if len(uv) == 1 and uv[0] == '':
             self.ignore_vert_code = True
+        return df
+
+    def search_all_files(self, update_file_convention=True):
+        """Search all valid model files for this model
+
+        This method browses the data directory and finds all valid files, that
+        is, file that are named according to one of the aerocom file naming
+        conventions. The file list is stored in :attr:`files`.
+
+        Note
+        ----
+        It is presumed, that naming conventions of files in
+        the data directory are not mixed but all correspond to either of the
+        conventions defined in
+
+        Parameters
+        ----------
+        update_file_convention : bool
+            if True, the first file in `data_dir` is used to identify the
+            file naming convention (cf. :class:`FileConventionRead`)
+
+        Raises
+        ------
+        DataCoverageError
+            if no valid files could be found
+        """
+        if self.data_dir is None:
+            raise AttributeError('please set data_dir first')
+
+
+        # get all files with correct ending
+        files = glob(f'{self.data_dir}/*{self.file_type}')
+        if len(files) == 0:
+            print_log.warning(
+                f'No files of type {self.file_type} could be found in current '
+                f'data directory (data_dir={os.path.abspath(self.data_dir)}')
+            return
+
+        if update_file_convention:
+            # Check if the found file has a naming according the aerocom conventions
+            # and set the convention for all files (maybe this need to be
+            # updated in case there can be more than one file naming convention
+            # within one model directory)
+            try:
+                self._update_file_convention(files)
+            except FileNotFoundError as e:
+                print_log.warning(repr(e))
+                return
+
+        result = self._evaluate_fileinfo(files)
+        df = self._fileinfo_to_dataframe(result)
         self.file_info = df
 
         if len(df) == 0:
-            raise DataCoverageError(f'No files could be found for {self.data_id}')
+            raise DataCoverageError(
+                f'No valid files could be found for {self.data_id}'
+                )
 
     def filter_files(self, var_name=None, ts_type=None, start=None, stop=None,
                      experiment=None, vert_which=None, is_at_stations=False,
@@ -1628,6 +1581,7 @@ class ReadGridded(object):
         elif not isnumeric(constraint['filter_val']):
             raise ValueError('Need numerical filter value')
 
+
     def apply_read_constraint(self, data, constraint,
                               **kwargs):
         """
@@ -1676,7 +1630,7 @@ class ReadGridded(object):
         if 'new_val' in constraint:
             new_val = constraint['new_val']
         else:
-            new_val = np.nan #np.ma.masked
+            new_val = np.nan
 
         operator_fun = self.CONSTRAINT_OPERATORS[constraint['operator']]
 
@@ -1689,23 +1643,38 @@ class ReadGridded(object):
         if not other_data.shape == data.shape:
             raise ValueError('Failed to apply filter. Shape mismatch')
 
-        # needs both data objects to be loaded into memory
-        #other_data._ensure_is_masked_array()
-        #data._ensure_is_masked_array()
+        other_arr = other_data.to_xarray()
+        arr = data.to_xarray()
+        if not other_arr.dims==arr.dims:
+            from pyaerocom.exceptions import DataDimensionError
+            raise DataDimensionError('Mismatch in dimensions')
+        for dim in arr.dims:
+            same_vals = (arr[dim].values==other_arr[dim].values).all()
+            if not same_vals:
+                if dim == 'time':
+                    other_arr[dim] = arr[dim]
+                else:
+                    raise ValueError()
 
-        other_arr = other_data.cube.core_data()
-        arr = data.cube.core_data()
 
         # select all grid points where conition is fulfilled
         mask = operator_fun(other_arr,
                             constraint['filter_val'])
 
-        # set values to NaN where condition is fulfilled
-
-        arr = np.where(mask, new_val, arr)
+        # set values to new_val where condition is fulfilled
+        filtered = xr.where(mask, new_val, arr)
 
         # overwrite data in cube with the filtered data
-        data.cube.data = arr
+        outcube = filtered.to_iris()
+        outcube.var_name = data.var_name
+        outcube.units = data.units
+        outcube.attributes = data.cube.attributes
+        outcube_dims = outcube.dim_coords
+        for i, name in enumerate(data.dimcoord_names):
+            _name = outcube_dims[i].name()
+            outcube.remove_coord(_name)
+            outcube.add_dim_coord(data.cube.coord(name), i)
+        data.cube = outcube
         return data
 
     def _try_read_var(self, var_name, start, stop,
@@ -1859,8 +1828,7 @@ class ReadGridded(object):
         return tuple(data)
 
 
-    def _load_files(self, files, var_name, perform_fmt_checks=None,
-                    **kwargs):
+    def _load_files(self, files, var_name, perform_fmt_checks=None):
         """Load list of files containing variable to read into Cube instances
 
         Parameters
@@ -1872,8 +1840,6 @@ class ReadGridded(object):
         perform_fmt_checks : bool
             if True, the loaded data is checked for consistency with
             AeroCom default requirements.
-        **kwargs
-            additional keyword args parsed to :func:`load_cubes_custom`.
 
         Returns
         -------
@@ -1882,9 +1848,12 @@ class ReadGridded(object):
         list
             list containing corresponding filenames of loaded cubes
         """
-        cubes, loaded_files = load_cubes_custom(files, var_name,
-                                                perform_fmt_checks=perform_fmt_checks,
-                                                **kwargs)
+        cubes, loaded_files = load_cubes_custom(
+            files=files,
+            var_name=var_name,
+            file_convention=self.file_convention,
+            perform_fmt_checks=perform_fmt_checks,
+            )
 
         if len(loaded_files) == 0:
             raise IOError("None of the input files could be loaded in {}"
@@ -1975,7 +1944,6 @@ class ReadGridded(object):
                            concatenated=is_concat,
                            convert_unit_on_init=try_convert_units,
                            **meta)
-        data.reader = self
         # crop cube in time (if applicable)
         if not start == 9999:
             try:
@@ -1986,6 +1954,7 @@ class ReadGridded(object):
                                           .format(data, start, stop))
         if rename_var is not None:
             data.var_name = rename_var
+        data.reader = self
         return data
 
     def _check_crop_time(self, data, start, stop):
@@ -2078,7 +2047,8 @@ class ReadGridded(object):
 if __name__=="__main__":
     import pyaerocom as pya
 
-    reader = ReadGridded('EMEP.cams50.u3all')
+    reader = ReadGridded('NorESM2-NHIST_f19_tn14_20190710')
     print(reader)
 
-    data = reader.read_var('wetoxs', try_convert_units=True)
+    data = reader.read_var('od550aer', start=9999)
+is_3d = lambda var_name: True if '3d' in var_name.lower() else False
\ No newline at end of file
